@@ -10,6 +10,7 @@ from neuromation.api import Client, ResourceNotFound
 from operator import attrgetter, itemgetter
 from rich import box
 from rich.console import Console
+from rich.panel import Panel
 from rich.table import Table
 from types import TracebackType
 from typing import AbstractSet, Dict, List, Mapping, Optional, Tuple, Type
@@ -22,7 +23,7 @@ from .commands import CmdProcessor
 from .config_loader import BatchLocalCL
 from .context import EMPTY_ROOT, EarlyBatch, RunningBatchFlow
 from .parser import ConfigDir
-from .storage import Attempt, Bake, BatchStorage, FinishedTask
+from .storage import Attempt, Bake, FinishedTask, Storage
 from .types import FullID, LocalPath, TaskStatus
 from .utils import TERMINATED_TASK_STATUSES, fmt_datetime, run_subproc
 
@@ -52,7 +53,7 @@ class BatchRunner(AsyncContextManager["BatchRunner"]):
         config_dir: ConfigDir,
         console: Console,
         client: Client,
-        storage: BatchStorage,
+        storage: Storage,
     ) -> None:
         self._config_dir = config_dir
         self._console = console
@@ -95,14 +96,10 @@ class BatchRunner(AsyncContextManager["BatchRunner"]):
     ) -> ExecutorData:
         # batch_name is a name of yaml config inside self._workspace / .neuro
         # folder without the file extension
-        self._console.print(f"[bright_black]neuromation=={neuromation.__version__}")
-        self._console.print(f"[bright_black]neuro-extras=={neuro_extras.__version__}")
-        self._console.print(f"[bright_black]neuro-flow=={neuro_flow.__version__}")
-        self._console.print(
-            f"Use config file {self.config_loader.flow_path(batch_name)}"
-        )
-
-        self._console.print("Check config... ", end="")
+        self._console.log(f"[bright_black]neuromation=={neuromation.__version__}")
+        self._console.log(f"[bright_black]neuro-extras=={neuro_extras.__version__}")
+        self._console.log(f"[bright_black]neuro-flow=={neuro_flow.__version__}")
+        self._console.log(f"Use config file {self.config_loader.flow_path(batch_name)}")
 
         # Check that the yaml is parseable
         flow = await RunningBatchFlow.create(self.config_loader, batch_name, params)
@@ -114,9 +111,11 @@ class BatchRunner(AsyncContextManager["BatchRunner"]):
 
         graphs = await self._build_graphs(flow)
 
-        self._console.print("[green]ok")
+        self._console.log(
+            "Check config... [green]ok[/green]",
+        )
 
-        self._console.print("Create bake")
+        self._console.log("Create bake...")
         config_meta, configs = await self.config_loader.collect_configs(batch_name)
         bake = await self._storage.create_bake(
             flow.project_id,
@@ -126,7 +125,7 @@ class BatchRunner(AsyncContextManager["BatchRunner"]):
             graphs=graphs,
             params=params,
         )
-        self._console.print(
+        self._console.log(
             f"Bake [b]{bake.bake_id}[/b] of project [b]{bake.project}[/b] is created"
         )
 
@@ -165,13 +164,13 @@ class BatchRunner(AsyncContextManager["BatchRunner"]):
         self,
         data: ExecutorData,
     ) -> None:
-        executor = await LocalsBatchExecutor.create(
+        async with LocalsBatchExecutor.create(
             self._console,
             data,
             self._client,
             self._storage,
-        )
-        await executor.run()
+        ) as executor:
+            await executor.run()
 
     async def bake(
         self,
@@ -179,21 +178,29 @@ class BatchRunner(AsyncContextManager["BatchRunner"]):
         local_executor: bool = False,
         params: Optional[Mapping[str, str]] = None,
     ) -> None:
+        self._console.print(
+            Panel(f"[bright_blue]Bake [b]{batch_name}[/b]", padding=1),
+            justify="center",
+        )
         data = await self._setup_exc_data(batch_name, params)
         await self._run_bake(data, local_executor)
 
     async def _run_bake(self, data: ExecutorData, local_executor: bool) -> None:
+        self._console.rule("Run local actions")
         await self._run_locals(data)
+        self._console.rule("Run main actions")
         if local_executor:
-            self._console.print(f"[bright_black]Using local executor")
+            self._console.log(f"[bright_black]Using local executor")
             await self.process(data)
         else:
-            self._console.print(f"[bright_black]Starting remote executor")
+            self._console.log(f"[bright_black]Starting remote executor")
             param = data.serialize()
             await run_subproc(
                 "neuro",
                 "run",
-                "--restart=on-failure",
+                # TODO: re-enable restarting following issue is fixed:
+                # https://github.com/neuro-inc/platform-client-python/issues/1797
+                # "--restart=on-failure",
                 "--pass-config",
                 EXECUTOR_IMAGE,
                 "neuro-flow",
@@ -206,10 +213,10 @@ class BatchRunner(AsyncContextManager["BatchRunner"]):
         self,
         data: ExecutorData,
     ) -> None:
-        executor = await BatchExecutor.create(
+        async with BatchExecutor.create(
             self._console, data, self._client, self._storage
-        )
-        await executor.run()
+        ) as executor:
+            await executor.run()
 
     def get_bakes(self) -> AsyncIterator[Bake]:
         return self._storage.list_bakes(self.project)
@@ -452,17 +459,24 @@ class BatchRunner(AsyncContextManager["BatchRunner"]):
         from_failed: bool = True,
     ) -> ExecutorData:
         bake = await self._storage.fetch_bake_by_id(self.project, bake_id)
+        last_attempt = await self._storage.find_attempt(bake, -1)
         attempt = await self._storage.find_attempt(bake, attempt_no)
         if attempt.result not in TERMINATED_TASK_STATUSES:
             raise click.BadArgumentUsage(
                 f"Cannot re-run still running attempt #{attempt.number} "
                 f"of {bake.bake_id}."
             )
-        if attempt.number >= 99:
+        if attempt.result == TaskStatus.SUCCEEDED and from_failed:
+            raise click.BadArgumentUsage(
+                f"Cannot re-run successful attempt #{attempt.number} "
+                f"of {bake.bake_id} with `--from-failed` flag set.\n"
+                "Hint: Try adding --no-from-failed to restart bake from the beginning."
+            )
+        if last_attempt.number >= 99:
             raise click.BadArgumentUsage(
                 f"Cannot re-run {bake.bake_id}, the number of attempts exceeded."
             )
-        new_att = await self._storage.create_attempt(bake, attempt.number + 1)
+        new_att = await self._storage.create_attempt(bake, last_attempt.number + 1)
         if from_failed:
             graphs = bake.graphs
             handled = set()  # a set of succesfully finished and not cached tasks
