@@ -9,7 +9,7 @@ from typing import Callable, Generic, List, Optional, Sequence, Tuple, TypeVar,
 from neuro_flow.batch_runner import BatchRunner
 from neuro_flow.cli.root import Root
 from neuro_flow.live_runner import LiveRunner
-from neuro_flow.storage import BatchFSStorage, BatchStorage, NeuroStorageFS
+from neuro_flow.storage import FSStorage, NeuroStorageFS, Storage
 
 
 if sys.version_info >= (3, 7):
@@ -68,14 +68,21 @@ class LiveJobType(AsyncType[str]):
     ) -> str:
         return value
 
-    async def async_complete(
+    async def async_complete(  # type: ignore[return]
         self,
         root: Root,
         ctx: click.Context,
         args: Sequence[str],
         incomplete: str,
     ) -> List[Tuple[str, Optional[str]]]:
-        async with LiveRunner(root.config_dir, root.console) as runner:
+        async with AsyncExitStack() as stack:
+            client = await stack.enter_async_context(api_get())
+            storage: Storage = await stack.enter_async_context(
+                FSStorage(NeuroStorageFS(client))
+            )
+            runner = await stack.enter_async_context(
+                LiveRunner(root.config_dir, root.console, client, storage)
+            )
             variants = list(runner.flow.job_ids)
             if self._allow_all:
                 variants += ["ALL"]
@@ -103,7 +110,7 @@ class LiveJobSuffixType(AsyncType[str]):
     ) -> str:
         return value
 
-    async def async_complete(
+    async def async_complete(  # type: ignore[return]
         self,
         root: Root,
         ctx: click.Context,
@@ -111,7 +118,14 @@ class LiveJobSuffixType(AsyncType[str]):
         incomplete: str,
     ) -> List[Tuple[str, Optional[str]]]:
         job_id = self._args_to_job_id(args)
-        async with LiveRunner(root.config_dir, root.console) as runner:
+        async with AsyncExitStack() as stack:
+            client = await stack.enter_async_context(api_get())
+            storage: Storage = await stack.enter_async_context(
+                FSStorage(NeuroStorageFS(client))
+            )
+            runner = await stack.enter_async_context(
+                LiveRunner(root.config_dir, root.console, client, storage)
+            )
             return [
                 (suffix, None)
                 for suffix in await runner.list_suffixes(job_id)
@@ -137,14 +151,21 @@ class LiveImageType(AsyncType[str]):
     ) -> str:
         return value
 
-    async def async_complete(
+    async def async_complete(  # type: ignore[return]
         self,
         root: Root,
         ctx: click.Context,
         args: Sequence[str],
         incomplete: str,
     ) -> List[Tuple[str, Optional[str]]]:
-        async with LiveRunner(root.config_dir, root.console) as runner:
+        async with AsyncExitStack() as stack:
+            client = await stack.enter_async_context(api_get())
+            storage: Storage = await stack.enter_async_context(
+                FSStorage(NeuroStorageFS(client))
+            )
+            runner = await stack.enter_async_context(
+                LiveRunner(root.config_dir, root.console, client, storage)
+            )
             variants = [
                 image
                 for image, image_ctx in runner.flow.images.items()
@@ -173,14 +194,21 @@ class LiveVolumeType(AsyncType[str]):
     ) -> str:
         return value
 
-    async def async_complete(
+    async def async_complete(  # type: ignore[return]
         self,
         root: Root,
         ctx: click.Context,
         args: Sequence[str],
         incomplete: str,
     ) -> List[Tuple[str, Optional[str]]]:
-        async with LiveRunner(root.config_dir, root.console) as runner:
+        async with AsyncExitStack() as stack:
+            client = await stack.enter_async_context(api_get())
+            storage: Storage = await stack.enter_async_context(
+                FSStorage(NeuroStorageFS(client))
+            )
+            runner = await stack.enter_async_context(
+                LiveRunner(root.config_dir, root.console, client, storage)
+            )
             variants = [
                 volume.id
                 for volume in runner.flow.volumes.values()
@@ -253,8 +281,8 @@ class BakeType(AsyncType[str]):
         variants = []
         async with AsyncExitStack() as stack:
             client = await stack.enter_async_context(api_get())
-            storage: BatchStorage = await stack.enter_async_context(
-                BatchFSStorage(NeuroStorageFS(client))
+            storage: Storage = await stack.enter_async_context(
+                FSStorage(NeuroStorageFS(client))
             )
             runner: BatchRunner = await stack.enter_async_context(
                 BatchRunner(root.config_dir, root.console, client, storage)
@@ -307,8 +335,8 @@ class BakeTaskType(AsyncType[str]):
         attempt_no = self._args_to_attempt(args)
         async with AsyncExitStack() as stack:
             client = await stack.enter_async_context(api_get())
-            storage: BatchStorage = await stack.enter_async_context(
-                BatchFSStorage(NeuroStorageFS(client))
+            storage: Storage = await stack.enter_async_context(
+                FSStorage(NeuroStorageFS(client))
             )
             runner: BatchRunner = await stack.enter_async_context(
                 BatchRunner(root.config_dir, root.console, client, storage)
