@@ -6,6 +6,7 @@ from neuromation.cli.log_formatter import ConsoleHandler
 from rich.console import Console
 from typing import Any, List, Optional
 
+import neuro_flow
 from neuro_flow.cli import batch, completion, images, live, storage
 from neuro_flow.parser import ConfigDir, find_workspace
 from neuro_flow.types import LocalPath
@@ -69,7 +70,9 @@ class MainGroup(click.Group):
         if show_traceback:
             LOG_ERROR = log.exception
 
-        ctx.obj = Root(config_dir=config_dir, console=Console(highlight=False))
+        ctx.obj = Root(
+            config_dir=config_dir, console=Console(highlight=False, log_path=False)
+        )
 
     def make_context(
         self,
@@ -133,6 +136,9 @@ class MainGroup(click.Group):
     default=False,
     required=False,
 )
+@click.version_option(
+    version=neuro_flow.__version__, message="neuro-flow package version: %(version)s"
+)
 def cli(
     config: Optional[str],
     fake_workspace: bool,
