@@ -38,9 +38,9 @@ from neuro_flow.types import LocalPath
 
 
 if sys.version_info >= (3, 7):  # pragma: no cover
-    from contextlib import asynccontextmanager  # noqa
+    from contextlib import asynccontextmanager
 else:
-    from async_generator import asynccontextmanager  # noqa
+    from async_generator import asynccontextmanager
 
 
 class ConfigLoader(abc.ABC):
