@@ -151,6 +151,7 @@ class CacheConf:
 class DefaultsConf:
     workdir: Optional[RemotePath] = None
     life_span: Optional[float] = None
+    schedule_timeout: Optional[float] = None
     preset: Optional[str] = None
 
 
@@ -164,6 +165,7 @@ class ExecUnit:
     name: Optional[str]
     image: str
     preset: Optional[str]
+    schedule_timeout: Optional[float]
     http_port: Optional[int]
     http_auth: Optional[bool]
     pass_config: Optional[bool]
@@ -539,12 +541,14 @@ async def setup_defaults_env_tags_ctx(
         workdir = await ast_defaults.workdir.eval(ctx)
         life_span = await ast_defaults.life_span.eval(ctx)
         preset = await ast_defaults.preset.eval(ctx)
+        schedule_timeout = await ast_defaults.schedule_timeout.eval(ctx)
     else:
         env = {}
         tags = set()
         workdir = None
         life_span = None
         preset = None
+        schedule_timeout = None
 
     tags.add(f"project:{_id2tag(ctx.flow.project_id)}")
     tags.add(f"flow:{_id2tag(ctx.flow.flow_id)}")
@@ -553,6 +557,7 @@ async def setup_defaults_env_tags_ctx(
         workdir=workdir,
         life_span=life_span,
         preset=preset,
+        schedule_timeout=schedule_timeout,
     )
     return defaults, env, tags
 
@@ -750,6 +755,23 @@ async def setup_matrix(ast_matrix: Optional[ast.Matrix]) -> Sequence[MatrixCtx]:
     return matrices
 
 
+async def setup_output_needs(
+    outputs: Optional[ast.BatchActionOutputs], known_tasks: AbstractSet[str]
+) -> AbstractSet[str]:
+    output_needs = set()
+    if outputs:
+        for need_expr in outputs.needs:
+            task_id = await need_expr.eval(EMPTY_ROOT)
+            if task_id not in known_tasks:
+                raise EvalError(
+                    f"Action does not contain task '{task_id}'",
+                    outputs._start,
+                    outputs._end,
+                )
+            output_needs.add(task_id)
+    return output_needs
+
+
 async def setup_cache(
     ctx: RootABC,
     base_cache: CacheConf,
@@ -794,6 +816,10 @@ class RunningLiveFlow:
     def job_ids(self) -> Iterable[str]:
         return sorted(self._ast_flow.jobs)
 
+    @property
+    def flow(self) -> FlowCtx:
+        return self._ctx.flow
+
     @property
     def tags(self) -> AbstractSet[str]:
         return self._ctx.tags
@@ -920,6 +946,9 @@ class RunningLiveFlow:
         life_span = (await job.life_span.eval(ctx)) or defaults.life_span
 
         preset = (await job.preset.eval(ctx)) or defaults.preset
+        schedule_timeout = (
+            await job.schedule_timeout.eval(ctx)
+        ) or defaults.schedule_timeout
         port_forward = []
         if job.port_forward is not None:
             port_forward = [await val.eval(ctx) for val in job.port_forward]
@@ -932,6 +961,7 @@ class RunningLiveFlow:
             name=await job.name.eval(ctx),
             image=await job.image.eval(ctx),
             preset=preset,
+            schedule_timeout=schedule_timeout,
             entrypoint=await job.entrypoint.eval(ctx),
             cmd=await job.cmd.eval(ctx),
             workdir=workdir,
@@ -1026,7 +1056,23 @@ class EarlyBatch:
 
         tasks = await EarlyTaskGraphBuilder(self._cl, prep_task.action.tasks).build()
 
-        return EarlyBatch(tasks, self._cl)
+        output_needs = await setup_output_needs(prep_task.action.outputs, tasks.keys())
+
+        return EarlyBatchAction(tasks, self._cl, output_needs)
+
+
+class EarlyBatchAction(EarlyBatch):
+    def __init__(
+        self,
+        tasks: Mapping[str, "BaseEarlyTask"],
+        config_loader: ConfigLoader,
+        output_needs: AbstractSet[str],
+    ):
+        super().__init__(tasks, config_loader)
+        self._output_needs = output_needs
+
+    async def get_output_needs(self) -> AbstractSet[str]:
+        return self._output_needs
 
 
 class RunningBatchBase(Generic[_T], EarlyBatch):
@@ -1132,7 +1178,9 @@ class RunningBatchBase(Generic[_T], EarlyBatch):
         life_span = (await prep_task.ast_task.life_span.eval(ctx)) or defaults.life_span
 
         preset = (await prep_task.ast_task.preset.eval(ctx)) or defaults.preset
-
+        schedule_timeout = (
+            await prep_task.ast_task.schedule_timeout.eval(ctx)
+        ) or defaults.schedule_timeout
         # Enable should be calculated using outer ctx for stateful calls
         enable = (await self.get_meta(real_id, needs, state)).enable
 
@@ -1142,6 +1190,7 @@ class RunningBatchBase(Generic[_T], EarlyBatch):
             name=(await prep_task.ast_task.name.eval(ctx)),
             image=await prep_task.ast_task.image.eval(ctx),
             preset=preset,
+            schedule_timeout=schedule_timeout,
             entrypoint=await prep_task.ast_task.entrypoint.eval(ctx),
             cmd=await prep_task.ast_task.cmd.eval(ctx),
             workdir=workdir,
@@ -1279,14 +1328,14 @@ class RunningBatchActionFlow(RunningBatchBase[BatchActionContext]):
         config_loader: ConfigLoader,
         defaults: DefaultsConf,
         action: ast.BatchAction,
+        output_needs: AbstractSet[str],
     ):
         super().__init__(ctx, tasks, config_loader, defaults)
         self._action = action
+        self._output_needs = output_needs
 
     async def get_output_needs(self) -> AbstractSet[str]:
-        if self._action.outputs:
-            return {await need.eval(self._ctx) for need in self._action.outputs.needs}
-        return set()
+        return self._output_needs
 
     async def calc_outputs(self, needs: NeedsCtx, *, is_cancelling: bool) -> DepCtx:
         if any(i.result == TaskStatus.FAILED for i in needs.values()):
@@ -1339,8 +1388,15 @@ class RunningBatchActionFlow(RunningBatchBase[BatchActionContext]):
             action_context, config_loader, cache, ast_action.tasks
         ).build()
 
+        output_needs = await setup_output_needs(ast_action.outputs, tasks.keys())
+
         return RunningBatchActionFlow(
-            action_context, tasks, config_loader, DefaultsConf(), ast_action
+            action_context,
+            tasks,
+            config_loader,
+            DefaultsConf(),
+            ast_action,
+            output_needs,
         )
 
 
