@@ -181,6 +181,8 @@ async def from_json(ctx: CallCtx, arg: str) -> TypeT:
 
 async def hash_files(ctx: CallCtx, *patterns: str) -> str:
     hasher = hashlib.new("sha256")
+    buffer = bytearray(16 * 1024 * 1024)  # 16 MB
+    view = memoryview(buffer)
     flow = ctx.root.lookup("flow")
     # emulate attr lookup
     workspace: LocalPath = cast(
@@ -191,11 +193,15 @@ async def hash_files(ctx: CallCtx, *patterns: str) -> str:
     )
     for pattern in patterns:
         for fname in sorted(workspace.glob(pattern)):
-            with fname.open("rb") as stream:
-                data = stream.read(256 * 1024)
-                if not data:
-                    break
-                hasher.update(data)
+            # On Windows the Python 3.6 glob() returns lower-cased filenames,
+            # resolve() restores the case.
+            relative_fname = fname.resolve().relative_to(workspace.resolve()).as_posix()
+            hasher.update(relative_fname.encode("utf-8"))
+            with fname.open("rb", buffering=0) as stream:
+                read = stream.readinto(buffer)  # type: ignore
+                while read:
+                    hasher.update(view[:read])
+                    read = stream.readinto(buffer)  # type: ignore
     return hasher.hexdigest()
 
 
@@ -216,6 +222,20 @@ async def upload(ctx: CallCtx, volume_ctx: ContainerT) -> ContainerT:
     return volume_ctx
 
 
+async def alower(ctx: CallCtx, arg: TypeT) -> str:
+    # Async version of lower(), async is required for the sake of uniformness.
+    if not isinstance(arg, str):
+        raise TypeError(f"lower() requires a str, got {arg!r}")
+    return arg.lower()
+
+
+async def aupper(ctx: CallCtx, arg: TypeT) -> str:
+    # Async version of lower(), async is required for the sake of uniformness.
+    if not isinstance(arg, str):
+        raise TypeError(f"upper() requires a str, got {arg!r}")
+    return arg.upper()
+
+
 def _check_has_needs(ctx: CallCtx, *, func_name: str) -> None:
     try:
         ctx.root.lookup("needs")
@@ -292,6 +312,8 @@ FUNCTIONS = _build_signatures(
     hash_files=hash_files,
     always=always,
     upload=upload,
+    lower=alower,
+    upper=aupper,
 )
 
 
@@ -847,18 +869,18 @@ class OptFloatExpr(FloatExprMixin, Expr[float]):
     type = float
 
 
-class OptLifeSpanExpr(OptFloatExpr):
+class OptTimeDeltaExpr(OptFloatExpr):
     RE = re.compile(r"^((?P<d>\d+)d)?((?P<h>\d+)h)?((?P<m>\d+)m)?((?P<s>\d+)s)?$")
 
     @classmethod
     def convert(cls, arg: Union[str, float]) -> float:
         try:
-            return super(cls, OptLifeSpanExpr).convert(arg)
+            return super(cls, OptTimeDeltaExpr).convert(arg)
         except (ValueError, SyntaxError):
             assert isinstance(arg, str)
             match = cls.RE.match(arg)
             if match is None:
-                raise ValueError(f"{arg!r} is not a life span")
+                raise ValueError(f"{arg!r} is not a time delta unit")
             td = datetime.timedelta(
                 days=int(match.group("d") or 0),
                 hours=int(match.group("h") or 0),
