@@ -42,11 +42,11 @@ from .expr import (
     OptBoolExpr,
     OptIdExpr,
     OptIntExpr,
-    OptLifeSpanExpr,
     OptLocalPathExpr,
     OptPythonExpr,
     OptRemotePathExpr,
     OptStrExpr,
+    OptTimeDeltaExpr,
     PortPairExpr,
     RemotePathExpr,
     SimpleIdExpr,
@@ -354,7 +354,7 @@ def parse_cache(ctor: BaseConstructor, node: yaml.MappingNode) -> ast.Cache:
         node,
         {
             "strategy": ast.CacheStrategy,
-            "life_span": OptLifeSpanExpr,
+            "life_span": OptTimeDeltaExpr,
         },
         ast.Cache,
     )
@@ -402,13 +402,23 @@ def parse_project_stream(stream: TextIO) -> ast.Project:
 
 
 def make_default_project(workspace_stem: str) -> ast.Project:
+    project_id = workspace_stem.replace("-", "_")
+    if not project_id.isidentifier():
+        raise ValueError(
+            f'Workspace directory name "{workspace_stem}" is invalid identifier'
+        )
+    if project_id == project_id.upper():
+        raise ValueError(
+            f'Workspace directory name "{workspace_stem}" is invalid '
+            "identifier, uppercase names are reserved for internal usage"
+        )
     return ast.Project(
         _start=Pos(0, 0, LocalPath("<default>")),
         _end=Pos(0, 0, LocalPath("<default>")),
         id=SimpleIdExpr(
             Pos(0, 0, LocalPath("<default>")),
             Pos(0, 0, LocalPath("<default>")),
-            workspace_stem.replace("-", "_"),
+            project_id,
         ),
     )
 
@@ -571,6 +581,7 @@ EXEC_UNIT = {
     "name": OptStrExpr,
     "image": StrExpr,
     "preset": OptStrExpr,
+    "schedule_timeout": OptTimeDeltaExpr,
     "entrypoint": OptStrExpr,
     "cmd": OptStrExpr,
     "bash": OptBashExpr,
@@ -579,7 +590,7 @@ EXEC_UNIT = {
     "env": SimpleMapping(StrExpr),
     "volumes": SimpleSeq(OptStrExpr),
     "tags": SimpleSeq(StrExpr),
-    "life_span": OptLifeSpanExpr,
+    "life_span": OptTimeDeltaExpr,
     "http_port": OptIntExpr,
     "http_auth": OptBoolExpr,
     "pass_config": OptBoolExpr,
@@ -756,8 +767,9 @@ def parse_flow_defaults(ctor: FlowLoader, node: yaml.MappingNode) -> ast.FlowDef
                 "tags": SimpleSeq(StrExpr),
                 "env": SimpleMapping(StrExpr),
                 "workdir": OptRemotePathExpr,
-                "life_span": OptLifeSpanExpr,
+                "life_span": OptTimeDeltaExpr,
                 "preset": OptStrExpr,
+                "schedule_timeout": OptTimeDeltaExpr,
             },
             ast.FlowDefaults,
         )
@@ -769,8 +781,9 @@ def parse_flow_defaults(ctor: FlowLoader, node: yaml.MappingNode) -> ast.FlowDef
                 "tags": SimpleSeq(StrExpr),
                 "env": SimpleMapping(StrExpr),
                 "workdir": OptRemotePathExpr,
-                "life_span": OptLifeSpanExpr,
+                "life_span": OptTimeDeltaExpr,
                 "preset": OptStrExpr,
+                "schedule_timeout": OptTimeDeltaExpr,
                 "fail_fast": OptBoolExpr,
                 "max_parallel": OptIntExpr,
                 "cache": None,
@@ -778,7 +791,7 @@ def parse_flow_defaults(ctor: FlowLoader, node: yaml.MappingNode) -> ast.FlowDef
             ast.BatchFlowDefaults,
         )
     else:
-        raise ValueError("Unknown kind {ctor._kind}")
+        raise ValueError(f"Unknown kind {ctor._kind}")
 
 
 FlowLoader.add_path_resolver(  # type: ignore
