@@ -16,7 +16,7 @@ class AlwaysT:
 
     def __new__(cls) -> "AlwaysT":
         if not hasattr(cls, "instance"):
-            cls.instance = super(AlwaysT, cls).__new__(cls)
+            cls.instance = super().__new__(cls)
         return cls.instance
 
 
